/////////////////////////////////////////////////////////////////////////////
// Name:        Menu.cpp
// Purpose:     The class to store a DVD Menu
// Author:      Alex Thuering
// Created:		04.11.2003
// RCS-ID:      $Id: Menu.cpp,v 1.52 2006/02/27 20:06:02 ntalex Exp $
// Copyright:   (c) Alex Thuering
// Licence:     GPL
/////////////////////////////////////////////////////////////////////////////

#include "Menu.h"
#include <wxVillaLib/SConv.h>
#include <wxVillaLib/utils.h>
#include <wxSVG/svg.h>
#include <wxXML/xmlhelpr.h>
#include <wx/mstream.h>

#define OBJECTS_DIR wxFindDataDirectory(_T("objects"))
#define BUTTONS_DIR wxFindDataDirectory(_T("buttons"))
#define BUTTON_MAX_COLORS 4

//////////////////////////////// MenuObject //////////////////////////////////

MenuObject::MenuObject(Menu* menu, wxString fileName, int x, int y, wxString param)
{
  m_svg = menu->GetSVG();
  m_use = NULL;
  m_symbol = NULL;
  
  m_fileName = fileName;
  
  m_button = false;
  m_actionTsi = -2; // current titleset
  m_actionPgci = 1;
  m_actionChapter = 0;
  
  m_defaultSize = true;
  m_defaultSizeWidth = m_defaultSizeHeight = 0;
  m_minSizeWidth = m_minSizeHeight = 0;
  
  if (fileName.length())
    Init(fileName, x, y, param);
}

MenuObject::~MenuObject()
{
  WX_CLEAR_ARRAY(m_params)
  if (m_use)
    m_use->GetParent()->RemoveChild(m_use);
  if (m_symbol)
    m_symbol->GetParent()->RemoveChild(m_symbol);
}

bool MenuObject::Init(wxString fileName, int x, int y, wxString param)
{
  m_fileName = fileName;
  wxXmlDocument xml;
  xml.Load(fileName);
  if (!xml.GetRoot())
    return false;
  wxXmlNode* root = xml.GetRoot();
  
  m_button = root->GetName() == wxT("button");
  m_title = XmlReadValue(root, wxT("title"));
  
  if (m_id.length()) // load button
  {
    m_symbol = (wxSVGSVGElement*) m_svg->GetElementById(wxT("s_") + m_id);
    m_use = (wxSVGUseElement*) m_svg->GetElementById(m_id);
    
    // fix old format (1.5b1 - 1.5b3)
    if (m_symbol && m_symbol->GetDtd() == wxSVG_SYMBOL_ELEMENT)
    {
      // replace symbol element with svg element
      wxSVGSymbolElement* symbol = (wxSVGSymbolElement*) m_symbol;
      m_symbol = new wxSVGSVGElement();
      m_symbol->SetId(symbol->GetId());
      m_symbol->SetViewBox(symbol->GetViewBox());
      m_symbol->SetPreserveAspectRatio(symbol->GetPreserveAspectRatio());
      wxXmlElement* child = symbol->GetChildren();
      while (child)
      {
        m_symbol->AddChild(child->CloneNode());
        child = child->GetNext();
      }
      wxXmlElement* parent = symbol->GetParent();
      parent->RemoveChild(symbol);
      parent->AddChild(m_symbol);
      // remove id prefix
      child = m_symbol->GetChildren();
      while (child)
      {
        wxSVGElement& elem = *(wxSVGElement*) child;
        if (elem.GetId().length())
            elem.SetId(elem.GetId().AfterFirst(wxT('_')).AfterFirst(wxT('_')));
        child = child->GetNext();
      }
    }
  }
  else // create new button
    m_id = GenerateId(m_button ? wxT("button") : wxT("obj"));
  
  wxXmlNode* svgNode = XmlFindNode(root, wxT("svg"));
  if (svgNode)
  {
    wxSVGDocument svg;
    LoadSVG(svg, svgNode);
    wxSVGSVGElement* root = svg.GetRootElement();
    if (root)
    {
      m_defaultSizeWidth = root->GetWidth().GetBaseVal();
      m_defaultSizeHeight = root->GetHeight().GetBaseVal();
      if (!m_symbol)
        AddSymol(m_id, root);
      if (!m_use)
        AddUse(m_id, x, y, m_defaultSizeWidth, m_defaultSizeHeight);
    }
  }
  
  wxXmlNode* paramsNode = XmlFindNode(root, wxT("parameters"));
  if (paramsNode)
  {
    wxXmlNode* child = paramsNode->GetChildren();
    while (child)
    {
      if (child->GetType() == wxXML_ELEMENT_NODE &&
          child->GetName() == wxT("parameter"))
      {
        MenuObjectParam* param = new MenuObjectParam;
        param->title = XmlReadValue(child, wxT("title"));
        param->name = XmlReadValue(child, wxT("name"));
        param->type = XmlReadValue(child, wxT("type"));
        param->element = XmlReadValue(child, wxT("element"));
        param->attribute = XmlReadValue(child, wxT("attribute"));
        param->changeable =
          XmlFindNode(child, wxT("changeable")) != NULL &&
          param->type == wxT("colour");
        m_params.Add(param);
        if (param->changeable)
        {
          param->changeable = false;
          param->normalColour = GetParamColour(param->name);
          wxCSSStyleDeclaration style;
          style.SetProperty(wxCSS_PROPERTY_STROKE,
            XmlReadValue(child, wxT("default-value/highlighted")));
          param->highlightedColour = style.GetStroke().GetRGBColor();
          style.SetProperty(wxCSS_PROPERTY_STROKE,
            XmlReadValue(child, wxT("default-value/selected")));
          param->selectedColour = style.GetStroke().GetRGBColor();
          param->changeable = true;
        }
      }
      child = child->GetNext();
    }
  }
  
  m_initParameter = XmlReadValue(root, wxT("init-parameter"));
  if (m_initParameter.length() && param.length())
    SetParam(m_initParameter, param);
  
  wxXmlNode* defaultSizeNode = XmlFindNode(root, wxT("default-size"));
  if (defaultSizeNode)
  {
    m_defaultSizeElement = XmlReadValue(defaultSizeNode, wxT("element"));
    long val;
    if (XmlReadValue(defaultSizeNode, wxT("width")).ToLong(&val))
      m_defaultSizeWidth = val;
    if (XmlReadValue(defaultSizeNode, wxT("height")).ToLong(&val))
      m_defaultSizeHeight = val;
  }
  
  wxXmlNode* minSizeNode = XmlFindNode(root, wxT("min-size"));
  if (minSizeNode)
  {
    m_minSizeElement = XmlReadValue(minSizeNode, wxT("element"));
    long val;
    if (XmlReadValue(minSizeNode, wxT("width")).ToLong(&val))
      m_minSizeWidth = val;
    if (XmlReadValue(minSizeNode, wxT("height")).ToLong(&val))
      m_minSizeHeight = val;
  }
    
  UpdateSize();
  
  return true;
}

bool MenuObject::LoadSVG(wxSVGDocument& svg, wxXmlNode* node)
{
  bool res;
  wxXmlDocument xml;
  xml.SetRoot(node->CloneNode());
  wxMemoryOutputStream output;
  xml.Save(output);
#if wxCHECK_VERSION(2,6,0)
  wxMemoryInputStream input(output);
  res = svg.Load(input);
#else
  char* data = new char[output.GetSize()];
  output.CopyTo(data, output.GetSize());
  wxMemoryInputStream input(data, output.GetSize());
  res = svg.Load(input);
  delete[] data;
#endif
  return res;
}

wxString MenuObject::GenerateId(wxString prefix)
{
  int i = 1;
  while(1)
  {
    wxString id = prefix + wxString::Format(wxT("%02d"), i);
    if (m_svg->GetElementById(id) == NULL)
      return id;
    i++;
  }
  return wxT("");
}

MenuObjectParam* MenuObject::GetObjectParam(wxString name)
{
  for (int i=0; i<(int)m_params.Count(); i++)
  {
    if (m_params[i]->name == name)
      return m_params[i];
  }
  return NULL;
}

MenuObjectParam* MenuObject::GetInitParam()
{
  if (!m_initParameter.length())
    return NULL;
  return GetObjectParam(m_initParameter);
}

wxSVGSVGElement* MenuObject::AddSymol(wxString id, wxSVGElement* content)
{
  m_symbol = new wxSVGSVGElement;
  m_symbol->SetId(wxT("s_") + id);
  m_svg->GetElementById(wxT("defs"))->AppendChild(m_symbol);
  if (content->GetDtd() == wxSVG_SVG_ELEMENT)
  {
    m_symbol->SetViewBox(((wxSVGSVGElement*) content)->GetViewBox());
    m_symbol->SetPreserveAspectRatio(((wxSVGSVGElement*) content)->GetPreserveAspectRatio());
  }
  else if (content->GetDtd() == wxSVG_SYMBOL_ELEMENT)
  {
    m_symbol->SetViewBox(((wxSVGSymbolElement*) content)->GetViewBox());
    m_symbol->SetPreserveAspectRatio(((wxSVGSymbolElement*) content)->GetPreserveAspectRatio());
  }
  wxXmlElement* child = content->GetChildren();
  while (child)
  {
    m_symbol->AppendChild(((wxSVGSVGElement*)child)->CloneNode());
    child = child->GetNext();
  }
  return m_symbol;
}

void MenuObject::AddUse(wxString id, int x, int y, int width, int height)
{
  m_use = new wxSVGUseElement;
  m_use->SetId(id);
  m_use->SetHref(wxT("#s_") + id);
  m_use->SetX(x);
  m_use->SetY(y);
  m_use->SetWidth(width);
  m_use->SetHeight(height);
  m_svg->GetRootElement()->AppendChild(m_use);
}

int MenuObject::GetX() { return m_use->GetX().GetBaseVal(); }
void MenuObject::SetX(int value) { m_use->SetX(value); }

int MenuObject::GetY() { return m_use->GetY().GetBaseVal(); }
void MenuObject::SetY(int value) { m_use->SetY(value); }

int MenuObject::GetWidth() { return m_use->GetWidth().GetBaseVal(); }
void MenuObject::SetWidth(int value) { m_use->SetWidth(value); }

int MenuObject::GetHeight() { return m_use->GetHeight().GetBaseVal(); }
void MenuObject::SetHeight(int value) { m_use->SetHeight(value); }

void MenuObject::FixSize(int& width, int& height)
{
  if (m_defaultSize)
  {
    width = m_defaultSizeWidth;
    height = m_defaultSizeHeight;
    if (m_defaultSizeElement.length())
    {
      wxSVGElement* elem = (wxSVGElement*) m_symbol->GetElementById(m_defaultSizeElement);
      if (elem)
      {
        if (elem->GetDtd() == wxSVG_IMAGE_ELEMENT)
        {
          width += ((wxSVGImageElement*)elem)->GetDefaultWidth();
          height += ((wxSVGImageElement*)elem)->GetDefaultHeight();
        }
        else
        {
          wxSVGRect bbox = wxSVGLocatable::GetElementBBox(elem);
          width += (int)bbox.GetWidth();
          height += (int)bbox.GetHeight();
        }
      }
    }
  }
  else
  {
    int w = m_minSizeWidth;
    int h = m_minSizeHeight;
    if (m_minSizeElement.length())
    {
      wxSVGElement* elem = (wxSVGElement*) m_symbol->GetElementById(m_minSizeElement);
      if (elem)
      {
        wxSVGRect bbox = wxSVGLocatable::GetElementBBox(elem);
        w += (int)bbox.GetWidth();
        h += (int)bbox.GetHeight();
      }
    }
    width = wxMax(width, w);
    height = wxMax(height, h);
  }
}

void MenuObject::UpdateSize()
{
  int width = GetWidth();
  int height = GetHeight();
  FixSize(width, height);
  SetWidth(width);
  SetHeight(height);
}

wxString MenuObject::GetParam(wxString name, MenuButtonState state)
{
  MenuObjectParam* param = GetObjectParam(name);
  if (!param)
    return wxT("");
  wxSVGElement* elem = (wxSVGElement*) m_symbol->GetElementById(param->element);
  if (!elem)
    return wxT("");
  if (param->attribute.length())
    return elem->GetAttribute(param->attribute);
  else if (elem->GetDtd() == wxSVG_TEXT_ELEMENT)
    return elem->GetChildren() ? elem->GetChildren()->GetContent() : wxT("");
  return wxT("");
}

void MenuObject::SetParam(wxString name, wxString value, MenuButtonState state)
{
  MenuObjectParam* param = GetObjectParam(name);
  if (!param)
    return;
  wxSVGElement* elem =(wxSVGElement*)  m_symbol->GetElementById(param->element);
  if (!elem)
    return;
  if (param->attribute.length())
    elem->SetAttribute(param->attribute, value);
  else if (elem->GetDtd() == wxSVG_TEXT_ELEMENT)
  {
    if (!elem->GetChildren())
      return;
    elem->GetChildren()->SetContent(value);
    ((wxSVGTextElement*)elem)->SetCanvasItem(NULL);
  }
}

int MenuObject::GetParamInt(wxString name, MenuButtonState state)
{
  long lval = 0;
  GetParam(name, state).ToLong(&lval);
  return lval;
}

void MenuObject::SetParamInt(wxString name, int value, MenuButtonState state)
{
  SetParam(name, wxString::Format(wxT("%d"), value), state);
}

wxFont MenuObject::GetParamFont(wxString name, MenuButtonState state)
{
  MenuObjectParam* param = GetObjectParam(name);
  if (!param)
    return wxFont(20, wxFONTFAMILY_DEFAULT, wxNORMAL, wxNORMAL, false);
  wxSVGElement* elem = (wxSVGElement*) m_symbol->GetElementById(param->element);
  if (!elem)
    return wxFont(20, wxFONTFAMILY_DEFAULT, wxNORMAL, wxNORMAL, false);
  
  int size = 20;
  double dval;
  if (elem->GetAttribute(wxT("font-size")).ToDouble(&dval))
    size = (int)dval;
  
  int style = wxFONTSTYLE_NORMAL;
  wxString styleStr = elem->GetAttribute(wxT("font-style"));
  if (styleStr == wxT("italic"))
    style = wxFONTSTYLE_ITALIC;
  else if (styleStr == wxT("oblique"))
    style = wxFONTSTYLE_SLANT;
  
  wxFontWeight weight = wxFONTWEIGHT_NORMAL;
  wxString weightStr = elem->GetAttribute(wxT("font-weight"));
  if (weightStr == wxT("bold"))
    weight = wxFONTWEIGHT_BOLD;
  if (weightStr == wxT("bolder"))
    weight = wxFONTWEIGHT_MAX;
  else if (weightStr == wxT("lighter"))
    weight = wxFONTWEIGHT_LIGHT;
  
  wxString faceName = elem->GetAttribute(wxT("font-family"));
  
  return wxFont(size, wxFONTFAMILY_DEFAULT, style, weight, false, faceName);
}

void MenuObject::SetParamFont(wxString name, wxFont value, MenuButtonState state)
{
  MenuObjectParam* param = GetObjectParam(name);
  if (!param)
    return;
  wxSVGElement* elem = (wxSVGElement*) m_symbol->GetElementById(param->element);
  if (!elem)
    return;
  
  elem->SetAttribute(wxT("font-size"),
    wxString::Format(wxT("%d"), value.GetPointSize()));
  
  wxString styleStr = wxT("normal");
  if (value.GetStyle() == wxFONTSTYLE_ITALIC)
    styleStr = wxT("italic");
  else if (value.GetStyle() == wxFONTSTYLE_SLANT)
    styleStr = wxT("oblique");
  elem->SetAttribute(wxT("font-style"), styleStr);
  
  wxString weightStr = wxT("normal");
  if (value.GetWeight() == wxFONTWEIGHT_BOLD)
    weightStr = wxT("bold");
  if (value.GetWeight() == wxFONTWEIGHT_MAX)
    weightStr = wxT("bolder");
  else if (value.GetWeight() == wxFONTWEIGHT_LIGHT)
    weightStr = wxT("lighter");
  elem->SetAttribute(wxT("font-weight"), weightStr);
  
  elem->SetAttribute(wxT("font-family"), value.GetFaceName());
}

wxColour MenuObject::GetParamColour(wxString name, MenuButtonState state)
{
  MenuObjectParam* param = GetObjectParam(name);
  if (!param)
    return wxT("");
  if (param->changeable)
  {
    switch (state)
    {
      case mbsNORMAL: return param->normalColour;
      case mbsHIGHLIGHTED: return param->highlightedColour;
      case mbsSELECTED: return param->selectedColour;
    }
  }
  wxSVGElement* elem = (wxSVGElement*) m_symbol->GetElementById(param->element);
  if (!elem)
    return wxT("");
  if (param->attribute.length())
  {
    const wxCSSStyleDeclaration& style = wxSVGStylable::GetElementStyle(*elem);
    const wxCSSValue& value = style.GetPropertyCSSValue(param->attribute);
    switch (value.GetCSSValueType())
    {
      case wxCSS_PRIMITIVE_VALUE:
        return ((wxCSSPrimitiveValue&)value).GetRGBColorValue();
      case wxCSS_SVG_COLOR:
      case wxCSS_SVG_PAINT:
        return ((wxSVGColor&)value).GetRGBColor();
      default:
        break;
    }
  }
  return wxColour();
}

void MenuObject::SetParamColour(wxString name, wxColour value, MenuButtonState state)
{
  MenuObjectParam* param = GetObjectParam(name);
  if (!param)
    return;
  if (param->changeable)
  {
    switch (state)
    {
      case mbsNORMAL: param->normalColour = value; break;
      case mbsHIGHLIGHTED: param->highlightedColour = value; break;
      case mbsSELECTED: param->selectedColour = value; break;
    }
  }
  if (state != mbsNORMAL)
    return;
  wxSVGElement* elem = (wxSVGElement*) m_symbol->GetElementById(param->element);
  if (!elem)
    return;
  if (param->attribute.length())
  {
    wxSVGPaint paint(value);
    elem->SetAttribute(param->attribute, paint.GetCSSText());
  }
}

void MenuObject::ToFront()
{
  m_svg->GetRootElement()->RemoveChild(m_use);
  m_svg->GetRootElement()->AppendChild(m_use);
}

void MenuObject::Forward()
{
   wxSVGElement* next = (wxSVGElement*) m_use->GetNextSibling();
  if (next && next->GetType() == wxXML_ELEMENT_NODE &&
              next->GetDtd() == wxSVG_USE_ELEMENT)
  {
    m_svg->GetRootElement()->RemoveChild(m_use);
    m_svg->GetRootElement()->InsertChild(m_use, next->GetNextSibling());
  }
}

void MenuObject::Backward()
{
  wxSVGElement* prev = (wxSVGElement*) m_use->GetPreviousSibling();
  if (prev && prev->GetType() == wxXML_ELEMENT_NODE &&
              prev->GetDtd() == wxSVG_USE_ELEMENT)
  {
    m_svg->GetRootElement()->RemoveChild(m_use);
    m_svg->GetRootElement()->InsertChild(m_use, prev);
  }
}

void MenuObject::ToBack()
{
  wxSVGElement* first = (wxSVGElement*) m_svg->GetRootElement()->GetFirstChild();
  while (first && (first->GetType() != wxXML_ELEMENT_NODE ||
                   first->GetDtd() != wxSVG_USE_ELEMENT))
    first = (wxSVGElement*) first->GetNextSibling();
  if (first && first != m_use)
  {
    m_svg->GetRootElement()->RemoveChild(m_use);
    m_svg->GetRootElement()->InsertChild(m_use, first);
  }
}

bool MenuObject::IsFirst()
{
  wxSVGElement* prev = (wxSVGElement*) m_use->GetPreviousSibling();
  return !prev || prev->GetType() != wxXML_ELEMENT_NODE ||
      prev->GetDtd() != wxSVG_USE_ELEMENT;
}

bool MenuObject::IsLast()
{
  wxSVGElement* next = (wxSVGElement*) m_use->GetNextSibling();
  return !next || next->GetType() != wxXML_ELEMENT_NODE ||
         next->GetDtd() != wxSVG_USE_ELEMENT;
}

wxImage MenuObject::GetImage(int maxWidth, int maxHeight)
{
  if (!m_use)
    return wxImage();
  m_svg->GetRootElement()->SetWidth(GetWidth());
  m_svg->GetRootElement()->SetHeight(GetHeight());
  return m_svg->Render(maxWidth, maxHeight);
}

wxString MenuObject::GetAction()
{
  wxString action = m_action;
  if (!action.length())
  {
	int tsi = GetActionTsi();
	bool menu = GetActionPgci()%2 == 0;
	int pgci = GetActionPgci()/2;
	action = _T("jump ");
	if (tsi == -1)
	  action += _T("vmgm ");
	else if (tsi >= 0)
	  action += wxString::Format(_T("titleset %d "), tsi+1);
	if (menu)
	  action += _T("menu");
	else
	  action += _T("title");
	if (!menu || pgci>0 || tsi == -2)
	  action += wxString::Format(_T(" %d"), pgci+1);
	if (GetActionChapter() > 0)
	  action += wxString::Format(_T(" chapter %d"), GetActionChapter()+1);
	action += _T(";");
  }
  return action;
}

wxXmlNode* MenuObject::GetXML(DVDFileType type, bool withSVG)
{
  wxString rootName = wxT("object");
  if (IsButton())
    rootName = wxT("button");
  wxXmlNode* rootNode = new wxXmlNode(wxXML_ELEMENT_NODE, rootName);
  switch (type)
  {
	case DVDSTYLER_XML:
      if (IsButton())
      {
        wxXmlNode* actionNode = new wxXmlNode(wxXML_ELEMENT_NODE, _T("action"));
        if (!GetCustomAction().length())
        {
          if (GetActionTsi()>-2)
            actionNode->AddProperty(_T("tsi"), wxString::Format(_T("%d"), GetActionTsi()));
          actionNode->AddProperty(_T("pgci"), wxString::Format(_T("%d"), GetActionPgci()));
          actionNode->AddProperty(_T("chapter"), wxString::Format(_T("%d"), GetActionChapter()));
        }
        else
          actionNode->AddChild(new wxXmlNode(wxXML_TEXT_NODE, wxEmptyString, GetCustomAction()));
        rootNode->AddChild(actionNode);
      }
      if (GetDirection(0).length() || GetDirection(1).length() ||
          GetDirection(2).length() || GetDirection(3).length())
      {
        wxXmlNode* directionNode = new wxXmlNode(wxXML_ELEMENT_NODE, _T("direction"));
        if (GetDirection(0).length())
          directionNode->AddProperty(_T("left"), GetDirection(0));
        if (GetDirection(1).length())
          directionNode->AddProperty(_T("right"), GetDirection(1));
        if (GetDirection(2).length())
          directionNode->AddProperty(_T("up"), GetDirection(2));
        if (GetDirection(3).length())
          directionNode->AddProperty(_T("down"), GetDirection(3));
        rootNode->AddChild(directionNode);
      }
      XmlWriteValue(rootNode, _T("filename"), GetFileName());
      for (int i=0; i<GetObjectParamsCount(); i++)
      {
        MenuObjectParam* param = GetObjectParam(i);
        if (param->changeable)
        {
          wxXmlNode* paramNode = new wxXmlNode(wxXML_ELEMENT_NODE, _T("parameter"));
          paramNode->AddProperty(_T("name"), param->name);
          paramNode->AddProperty(_T("normal"),
            wxSVGPaint(param->normalColour).GetCSSText());
          paramNode->AddProperty(_T("highlighted"),
            wxSVGPaint(param->highlightedColour).GetCSSText());
          paramNode->AddProperty(_T("selected"),
            wxSVGPaint(param->selectedColour).GetCSSText());
          rootNode->AddChild(paramNode);
        }
      }
      rootNode->AddProperty(_T("id"), GetId());
      if (IsDefaultSize())
        rootNode->AddProperty(_T("defSize"), _T("true"));
      break;
	case SPUMUX_XML:
    {
      int mwidth = (int)m_svg->GetRootElement()->GetWidth().GetBaseVal()-1;
      int mheight = (int)m_svg->GetRootElement()->GetHeight().GetBaseVal()-1;
	  rootNode->AddProperty(_T("name"), GetId());
	  rootNode->AddProperty(_T("x0"), wxString::Format(_T("%d"),
        GetX() > 0 ? GetX() : 0));
	  rootNode->AddProperty(_T("y0"), wxString::Format(_T("%d"),
        GetY() > 0 ? GetY() : 0));
	  rootNode->AddProperty(_T("x1"), wxString::Format(_T("%d"),
        GetX() + GetWidth() < mwidth ? GetX() + GetWidth() : mwidth));
	  rootNode->AddProperty(_T("y1"), wxString::Format(_T("%d"),
        GetY() + GetHeight() < mheight ? GetY() + GetHeight() : mheight));
	  if (GetDirection(0).length())
		rootNode->AddProperty(_T("left"), GetDirection(0));
	  if (GetDirection(1).length())
		rootNode->AddProperty(_T("right"), GetDirection(1));
	  if (GetDirection(2).length())
		rootNode->AddProperty(_T("up"), GetDirection(2));
	  if (GetDirection(3).length())
		rootNode->AddProperty(_T("down"), GetDirection(3));
	  break;
    }
	case DVDAUTHOR_XML:
	  rootNode->AddProperty(_T("name"), GetId());
	  rootNode->AddChild(new wxXmlNode(wxXML_TEXT_NODE, wxEmptyString, GetAction()));
	  break;
	default:
	  break;
  }
  if (withSVG) // used by copy & paste
  {
    rootNode->DeleteProperty(wxT("id"));
    wxXmlNode* svgNode = new wxXmlNode(wxXML_ELEMENT_NODE, _T("svg"));
    wxXmlNode* defsNode = new wxXmlNode(wxXML_ELEMENT_NODE, _T("defs"));
    wxSVGElement* symbol = (wxSVGElement*) m_symbol->CloneNode();
    symbol->SetId(wxT(""));
    defsNode->AddChild(symbol);
    svgNode->AddChild(defsNode);
    svgNode->AddChild(m_use->CloneNode());
    rootNode->AddChild(svgNode);
  }
  return rootNode;
}

bool MenuObject::PutXML(wxXmlNode* node)
{
  m_button = node->GetName() == wxT("button");
  
  long lval;
  wxString val;
    
  if (IsButton())
  {
    wxXmlNode* actionNode = XmlFindNodeSimple(node, _T("action"));
    if (actionNode)
    {
      if (actionNode->GetPropVal(_T("tsi"), &val) && val.ToLong(&lval))
        SetActionTsi(lval);
      if (actionNode->GetPropVal(_T("pgci"), &val) && val.ToLong(&lval))
        SetActionPgci(lval);
      if (actionNode->GetPropVal(_T("chapter"), &val) && val.ToLong(&lval))
        SetActionChapter(lval);
      SetCustomAction(XmlReadValue(node, _T("action")).Strip(wxString::both));
    }
    
    wxXmlNode* dirNode = XmlFindNodeSimple(node, _T("direction"));
    if (dirNode)
    {
      if (dirNode->GetPropVal(_T("left"), &val))
        SetDirection(0, val);
      if (dirNode->GetPropVal(_T("right"), &val))
        SetDirection(1, val);
      if (dirNode->GetPropVal(_T("up"), &val))
        SetDirection(2, val);
      if (dirNode->GetPropVal(_T("down"), &val))
        SetDirection(3, val);
    }
  }
  
  // deprecated
  int x = 0, y = 0;
  if (node->GetPropVal(_T("x"), &val) && val.ToLong(&lval))
    x = (int)(lval/m_svg->GetScale());
  if (node->GetPropVal(_T("y"), &val) && val.ToLong(&lval))
    y = (int)(lval/m_svg->GetScale());
  wxString fileName = XmlReadValue(node, _T("type"));
  if (fileName.length())
  {
    wxString text = XmlReadValue(node, _T("text"));
    return Init(BUTTONS_DIR + wxT("text.xml"), x, y, text);
  }
  
  fileName = XmlReadValue(node, _T("filename"));
  if (!fileName.length())
    return false;
  
  node->GetPropVal(_T("id"), &m_id);
  if (!node->GetPropVal(_T("defSize"), &val) ||
      (val != wxT("true") && val != wxT("1")))
    SetDefaultSize(false);
  
  // paste button
  wxXmlNode* svgNode = XmlFindNode(node, wxT("svg"));
  if (svgNode)
  {
    m_id = GenerateId(m_button ? wxT("button") : wxT("obj"));
    wxSVGDocument svg;
    LoadSVG(svg, svgNode);
    wxSVGSVGElement* root = svg.GetRootElement();
    if (root && XmlFindNode(root, wxT("defs")) && XmlFindNode(root, wxT("use")))
    {
      wxSVGElement* defsNode = (wxSVGElement*) XmlFindNode(root, wxT("defs"));
      wxXmlElement* child = defsNode->GetChildren();
      while (child)
      {
        AddSymol(m_id, (wxSVGElement*) child);
        child = child->GetNext();
      }
      wxSVGUseElement* useElem = (wxSVGUseElement*) XmlFindNode(root, wxT("use"));
      AddUse(m_id, useElem->GetX().GetBaseVal(), useElem->GetY().GetBaseVal(),
        useElem->GetWidth().GetBaseVal(), useElem->GetHeight().GetBaseVal());
    }
  }
  
  if (!Init(fileName, x, y))
    return false;
  
  wxXmlNode* child = node->GetChildren();
  while (child)
  {
    if (child->GetName() == wxT("parameter"))
    {
      wxString name;
      wxString normal;
      wxString selected;
      wxString highlighted;
      if (child->GetPropVal(wxT("name"), &name) &&
          child->GetPropVal(wxT("normal"), &normal) &&
          child->GetPropVal(wxT("highlighted"), &highlighted) &&
          child->GetPropVal(wxT("selected"), &selected))
      {
        wxCSSStyleDeclaration style;
        style.SetProperty(wxT("fill"), normal);
        SetParamColour(name, style.GetFill().GetRGBColor(), mbsNORMAL);
        style.SetProperty(wxT("fill"), highlighted);
        SetParamColour(name, style.GetFill().GetRGBColor(), mbsHIGHLIGHTED);
        style.SetProperty(wxT("fill"), selected);
        SetParamColour(name, style.GetFill().GetRGBColor(), mbsSELECTED);
      }
    }
    child = child->GetNext();
  }
  
  return true;
}

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
////////////////////////////////// Menu //////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

Menu::Menu(VideoFormat videoFormat)
{
  m_videoFormat = videoFormat;
  m_startTime = _T("00:00:00.00");
  m_transpColour = wxColour(8,8,8);
  
  m_svg = new wxSVGDocument;
  wxSVGSVGElement* svgElement = new wxSVGSVGElement;
  svgElement->SetWidth(GetResolution().GetWidth());
  svgElement->SetHeight(GetResolution().GetHeight());
  m_svg->AppendChild(svgElement);
  wxSVGDefsElement* defsElement = new wxSVGDefsElement;
  defsElement->SetId(wxT("defs"));
  svgElement->AppendChild(defsElement);
  
  SetBackgroundColour(wxColour(0,0,0));
}

Menu::~Menu()
{
  delete m_svg;
  WX_CLEAR_ARRAY(m_objects)
}

void Menu::SetVideoFormat(VideoFormat format)
{
  m_videoFormat = format;
  // svg root element
  m_svg->GetRootElement()->SetWidth(GetResolution().GetWidth());
  m_svg->GetRootElement()->SetHeight(GetResolution().GetHeight());
  // background
  wxSVGElement* bgElement = m_svg->GetElementById(wxT("background"));
  if (bgElement)
  {
    if (bgElement->GetDtd() == wxSVG_RECT_ELEMENT)
    {
      ((wxSVGRectElement*)bgElement)->SetWidth(GetResolution().GetWidth());
      ((wxSVGRectElement*)bgElement)->SetHeight(GetResolution().GetHeight());
    }
    else if (bgElement->GetDtd() == wxSVG_IMAGE_ELEMENT)
    {
      ((wxSVGImageElement*)bgElement)->SetWidth(GetResolution().GetWidth());
      ((wxSVGImageElement*)bgElement)->SetHeight(GetResolution().GetHeight());
    }
  }
}

wxArrayString Menu::GetVideoFormatStrings()
{
  wxArrayString formats;
  formats.Add(_T("PAL 720x576"));
  formats.Add(_T("NTSC 720x480"));
  return formats;
}

wxSize Menu::GetResolution()
{
  if (m_videoFormat == vfNTSC)
	return wxSize(720,480);
  return wxSize(720,576); // vfPAL
}

void Menu::SetBackground(wxString fileName)
{
  bool image = wxImage::FindHandler(fileName.AfterLast(wxT('.')).Lower(), -1);
  
  // test if element exists
  wxSVGElement* bgElement = m_svg->GetElementById(wxT("background"));
  if (!bgElement ||
      (image && bgElement->GetDtd() != wxSVG_IMAGE_ELEMENT) ||
      (!image && bgElement->GetDtd() != wxSVG_VIDEO_ELEMENT))
  {
    if (bgElement) // remove old
      bgElement->GetParent()->RemoveChild(bgElement);
    if (image) // create image element
    {
      bgElement = new wxSVGImageElement;
      ((wxSVGImageElement*) bgElement)->SetWidth(GetResolution().GetWidth());
      ((wxSVGImageElement*) bgElement)->SetHeight(GetResolution().GetHeight());
    }
    else // create video element
    {
      bgElement = new wxSVGVideoElement;
      ((wxSVGVideoElement*) bgElement)->SetWidth(GetResolution().GetWidth());
      ((wxSVGVideoElement*) bgElement)->SetHeight(GetResolution().GetHeight());
    }
    bgElement->SetId(wxT("background"));
    m_svg->GetRootElement()->InsertChild(bgElement,
      m_svg->GetRootElement()->GetChildren());
  }
  
  // set href
  if (image)
    ((wxSVGImageElement*) bgElement)->SetHref(fileName);
  else
    ((wxSVGVideoElement*) bgElement)->SetHref(fileName);
}

wxString Menu::GetBackground()
{
  wxSVGElement* bgElement =
    (wxSVGElement*) m_svg->GetElementById(wxT("background"));
  if (bgElement)
  {
    if (bgElement->GetDtd() == wxSVG_IMAGE_ELEMENT)
      return ((wxSVGImageElement*)bgElement)->GetHref();
    else if (bgElement->GetDtd() == wxSVG_VIDEO_ELEMENT)
      return ((wxSVGVideoElement*)bgElement)->GetHref();
  }
  return wxT("");
}

void Menu::SetBackgroundColour(wxColour colour)
{
  wxSVGElement* bgElement = m_svg->GetElementById(wxT("background"));
  wxSVGRectElement* bgRect = (wxSVGRectElement*) bgElement;
  if (!bgElement || bgElement->GetDtd() != wxSVG_RECT_ELEMENT)
  {
    if (bgElement)
      bgElement->GetParent()->RemoveChild(bgElement);
    bgRect = new wxSVGRectElement;
    bgRect->SetId(wxT("background"));
    bgRect->SetWidth(GetResolution().GetWidth());
    bgRect->SetHeight(GetResolution().GetHeight());
    m_svg->GetRootElement()->InsertChild(bgRect,
      m_svg->GetRootElement()->GetChildren());
  }
  bgRect->SetFill(wxSVGPaint(colour.Red(), colour.Green(), colour.Blue()));
}

wxColour Menu::GetBackgroundColour()
{
  wxSVGRectElement* bgRect =
    (wxSVGRectElement*) m_svg->GetElementById(wxT("background"));
  if (bgRect && bgRect->GetDtd() == wxSVG_RECT_ELEMENT)
    return bgRect->GetFill().GetRGBColor();
  return wxColour(0,0,0);
}

bool Menu::HasVideoBackground()
{
  wxSVGElement* bgElement = m_svg->GetElementById(wxT("background"));
  return bgElement && bgElement->GetDtd() == wxSVG_VIDEO_ELEMENT;
}

void Menu::SetTransparentColor()
{
  m_transpColour = wxColour(0,0,0);
  wxImageHistogram h1, h2, h3;
  GetImage(mdBUTTONS_NORMAL).ComputeHistogram(h1);
  GetImage(mdBUTTONS_HIGHLIGHTED).ComputeHistogram(h2);
  GetImage(mdBUTTONS_SELECTED).ComputeHistogram(h3);
  for (int i = 1; i< 30; i++)
  {
	m_transpColour = wxColour(i*8, i*8, i*8);
	unsigned long colour = (i*8<<16) + (i*8<<8) + i*8;
	if (h1.find(colour) == h1.end() &&
		h2.find(colour) == h2.end() &&
		h3.find(colour) == h3.end())
	  break;
  }
}

bool Menu::IsDefElement(wxSVGElement* element)
{
  if (!element || element->GetDtd() == wxSVG_SVG_ELEMENT)
    return false;
  if (element->GetDtd() == wxSVG_DEFS_ELEMENT)
    return true;
  return IsDefElement((wxSVGElement*) element->GetParent());
}

void Menu::RemoveChangeable(wxSVGElement* element, MenuObject* obj)
{
  wxSVGElement* child = (wxSVGElement*) element->GetChildren();
  while (child)
  {
    wxSVGElement* elem = child;
    child = (wxSVGElement*) child->GetNext();
    if (elem->GetType() != wxXML_ELEMENT_NODE)
      continue;
    // don't remove def elements
    if (IsDefElement(elem))
      continue;
    // check if elem changeable
    if (elem->GetId().length())
    {
      // check if element has changeable attributes
      bool isChangeable = false;
      for (int i=0; i<obj->GetObjectParamsCount(); i++)
      {
        MenuObjectParam* param = obj->GetObjectParam(i);
        if (param->element == elem->GetId() && param->changeable)
        {
          isChangeable = true;
          break;
        }
      }
      if (isChangeable)
      {
        // todo: don't remove if it has not changeable attributes, but
        // remove all changeable attributes: SetFill(none),SetStroke(none)
        elem->GetParent()->RemoveChild(elem);
        continue;
      }
    }
    // remove changeable children
    RemoveChangeable(elem, obj);
  }
}

bool Menu::RemoveNotChangeable(wxSVGElement* element, MenuObject* obj)
{
  wxSVGElement* child = (wxSVGElement*) element->GetChildren();
  while (child)
  {
    wxSVGElement* elem = child;
    child = (wxSVGElement*) child->GetNext();
    if (elem->GetType() != wxXML_ELEMENT_NODE)
    {
      elem->GetParent()->RemoveChild(elem);
      continue;
    }
    // don't remove def elements
    if (IsDefElement(elem))
      continue;
    // check if child changeable
    if (elem->GetId().length())
    {
      // check if element has changeable attributes
      bool isChangeable = false;
      for (int i=0; i<obj->GetObjectParamsCount(); i++)
      {
        MenuObjectParam* param = obj->GetObjectParam(i);
        if (param->element == elem->GetId() && param->changeable)
        {
          isChangeable = true;
          break;
        }
      }
      if (isChangeable)
      {
        // todo: remove not changeable attributes: SetFill(none),SetStroke(none)
        continue;
      }
    }
    // check if it has changeable children
    if (RemoveNotChangeable(elem, obj))
      elem->GetParent()->RemoveChild(elem);
  }
  return element->GetChildren() == NULL; // return if has no children
}

wxImage Menu::GetImage(MenuDrawType drawType, int width, int height)
{
  wxSVGDocument* svg = new wxSVGDocument();
  svg->AppendChild(m_svg->GetRootElement()->CloneNode());
  wxSVGElement* selRect = svg->GetElementById(wxT("selection"));
  if (selRect)
    selRect->GetParent()->RemoveChild(selRect);
  
  if (drawType == mdBACKGROUND)
  {
    for (int i=0; i<GetObjectsCount(); i++)
    {
      MenuObject* obj = GetObject(i);
      if (obj->IsButton())
      {
        wxSVGElement* elem = svg->GetElementById(wxT("s_") + obj->GetId());
        if (elem)
          RemoveChangeable(elem, obj);
      }
    }
  }
  else if (drawType == mdBUTTONS_NORMAL ||
           drawType == mdBUTTONS_HIGHLIGHTED ||
           drawType == mdBUTTONS_SELECTED)
  {
    wxSVGElement* bgElement = svg->GetElementById(wxT("background"));
    wxSVGRectElement* bgRect = (wxSVGRectElement*) bgElement;
    if (!bgElement || bgElement->GetDtd() != wxSVG_RECT_ELEMENT)
    {
      if (bgElement)
        bgElement->GetParent()->RemoveChild(bgElement);
      bgRect = new wxSVGRectElement;
      bgRect->SetId(wxT("background"));
      bgRect->SetWidth(GetResolution().GetWidth());
      bgRect->SetHeight(GetResolution().GetHeight());
      svg->GetRootElement()->InsertChild(bgRect,
        svg->GetRootElement()->GetChildren());
    }
    bgRect->SetFill(wxSVGPaint(m_transpColour.Red(), m_transpColour.Green(), m_transpColour.Blue()));
    for (int i=0; i<GetObjectsCount(); i++)
    {
      MenuObject* obj = GetObject(i);
      if (obj->IsButton())
      {
        wxSVGSVGElement* symbol = (wxSVGSVGElement*)
          svg->GetElementById(wxT("s_") + obj->GetId());
        if (symbol)
          RemoveNotChangeable(symbol, obj);
        for (int i=0; i<obj->GetObjectParamsCount(); i++)
        {
          MenuObjectParam* param = obj->GetObjectParam(i);
          if (param->changeable && drawType != mdBUTTONS_NORMAL)
          {
            wxSVGElement* elem = (wxSVGElement*)
              symbol->GetElementById(param->element);
            if (elem && param->attribute.length())
            {
              wxSVGPaint paint(drawType == mdBUTTONS_SELECTED ?
                param->selectedColour : param->highlightedColour);
              elem->SetAttribute(param->attribute, paint.GetCSSText());
            }
          }
        }
      }
      else
      {
        wxSVGElement* elem = svg->GetElementById(obj->GetId());
        if (elem && elem->GetParent())
          elem->GetParent()->RemoveChild(elem);
        elem = svg->GetElementById(wxT("s_") + obj->GetId());
        if (elem && elem->GetParent())
          elem->GetParent()->RemoveChild(elem);
      }
    }
  }
  
  wxImage image = svg->Render(width, height);
  delete svg;
  
  if (drawType != mdALL && drawType != mdBACKGROUND)
  {
    // make aliasing for buttons
    for (int i=0; i<GetObjectsCount(); i++)
    {
      MenuObject* obj = GetObject(i);
      if (!obj->IsButton())
        continue;
      // make palette
      wxArrayLong colors;
      for (int i=0; i<obj->GetObjectParamsCount(); i++)
      {
        MenuObjectParam* param = obj->GetObjectParam(i);
        if (param->changeable) 
        {
          wxColour color;
          switch(drawType)
          {
            case mdBUTTONS_NORMAL: color = param->normalColour; break;
            case mdBUTTONS_HIGHLIGHTED: color = param->highlightedColour; break;
            case mdBUTTONS_SELECTED: color = param->selectedColour; break;
            default: break;
          }
          if (color.Ok())
            colors.Add((color.Red()<<16) + (color.Green()<<8) + color.Blue());
        }
      }
      unsigned char r[colors.Count()], g[colors.Count()], b[colors.Count()];
      for (int i=0; i<(int)colors.Count(); i++)
      {
        unsigned long color = (unsigned long) colors[i];
        r[i] = (color>>16)&0xFF;
        g[i] = (color>>8)&0xFF;
        b[i] = color&0xFF;
      }
      wxPalette palette(colors.Count(), r, g, b);
      // apply palette
      unsigned char* data =
        image.GetData() + obj->GetY()*image.GetWidth()*3 + obj->GetX()*3;
      for (int y=0; y<obj->GetHeight(); y++)
      {
        for (int x=0; x<obj->GetWidth(); x++)
        {
          if (data[0] != m_transpColour.Red() ||
              data[1] != m_transpColour.Green() ||
              data[2] != m_transpColour.Blue())
          {
            int n = palette.GetPixel(data[0], data[1], data[2]);
            palette.GetRGB(n, data, data+1, data+2);
          }
          data+=3;
        }
        data += (image.GetWidth()-obj->GetWidth())*3;
      }
    }
	// count of colors must be < BUTTON_MAX_COLORS
    if (image.CountColours(BUTTON_MAX_COLORS+1)>BUTTON_MAX_COLORS)
    {
      wxImageHistogram histogram;
      image.ComputeHistogram(histogram);
      wxArrayLong colors;
      wxArrayLong values;
      wxImageHistogram::iterator it;
      for(it = histogram.begin(); it != histogram.end(); ++it )
      {
        colors.Add(it->first);
        values.Add(it->second.value);        
      }
      //sort
      bool flag = true;
      while (flag)
      {
        flag = false;
        for (int i=0; i<(int)colors.GetCount()-1; i++)
        if (values[i] < values[i+1])
        {
          long tmp = values[i];
          values[i] = values[i+1];
          values[i+1] = tmp;
          tmp = colors[i];
          colors[i] = colors[i+1];
          colors[i+1] = tmp;
          flag = true;
        }
      }
	  //todo: remove like colors
      //create Palette
      unsigned long bg_color = colors[0]; //transparent color
      unsigned char r[BUTTON_MAX_COLORS-1],
       g[BUTTON_MAX_COLORS-1], b[BUTTON_MAX_COLORS-1];
      for (int i=0; i<(int)BUTTON_MAX_COLORS-1; i++)
      {
        unsigned long color = (unsigned long) colors[i+1];
        r[i] = (color>>16)&0xFF;
        g[i] = (color>>8)&0xFF;
        b[i] = color&0xFF;
      }
      wxPalette palette(BUTTON_MAX_COLORS-1, r, g, b);
      //image.SetPalette(palette);
      unsigned char* data = image.GetData();
      for (int y=0; y<image.GetHeight(); y++)
      for (int x=0; x<image.GetWidth(); x++)
      {
        unsigned long color = (data[0]<<16) + (data[1]<<8) + data[2];
        if (color != bg_color)
        {
          int n = palette.GetPixel(data[0], data[1], data[2]);
          palette.GetRGB(n, data, data+1, data+2);
          //data[0] = 255; data[1] = 255; data[2] = 255;
        }
        data+=3;
      }
    }
  }
  
  return image;
}

MenuObject* Menu::GetObject(wxString id)
{
  for (int i=0; i<(int)m_objects.Count(); i++)
    if (m_objects[i]->GetId() == id)
      return m_objects[i];
  return NULL;
}

void Menu::ObjectToFront(wxString id)
{
  MenuObject* obj = GetObject(id);
  if (obj)
    obj->ToFront();
}

void Menu::ObjectForward(wxString id)
{
  MenuObject* obj = GetObject(id);
  if (obj)
    obj->Forward();
}

void Menu::ObjectBackward(wxString id)
{
  MenuObject* obj = GetObject(id);
  if (obj)
    obj->Backward();
}

void Menu::ObjectToBack(wxString id)
{
  MenuObject* obj = GetObject(id);
  if (obj)
    obj->ToBack();
}

bool Menu::IsFirstObject(wxString id)
{
  MenuObject* obj = GetObject(id);
  return obj && obj->IsFirst();
}

bool Menu::IsLastObject(wxString id)
{
  MenuObject* obj = GetObject(id);
  return obj && obj->IsLast();
}

void Menu::RemoveObject(wxString id)
{
  MenuObject* obj = GetObject(id);
  if (obj)
  {
    m_objects.Remove(obj);
    delete obj;
  }
}

wxString Menu::AddImage(wxString fileName, int x, int y)
{
  return AddObject(OBJECTS_DIR + wxT("/image.xml"), fileName, x, y);
}

wxString Menu::AddText(wxString text, int x, int y)
{
  return AddObject(OBJECTS_DIR + wxT("/text.xml"), text, x, y);
}

wxString Menu::AddObject(wxString fileName, wxString param, int x, int y)
{
  MenuObject* obj = new MenuObject(this, fileName, x, y, param);
  m_objects.Add(obj);
  return obj->GetId();
}

bool Menu::SaveSpumux(wxString fileName,
  wxString btFile, wxString hlFile, wxString selFile)
{
  wxXmlDocument xml;
  wxXmlNode* root = new wxXmlNode(wxXML_ELEMENT_NODE, _T("subpictures"));
  wxXmlNode* streamNode = new wxXmlNode(wxXML_ELEMENT_NODE, _T("stream"));
  wxXmlNode* spuNode = new wxXmlNode(wxXML_ELEMENT_NODE, _T("spu"));
  if (GetStartTime().Length())
	spuNode->AddProperty(_T("start"), GetStartTime());
  if (GetEndTime().Length())
	spuNode->AddProperty(_T("end"), GetEndTime());
  spuNode->AddProperty(_T("image"), btFile);
  spuNode->AddProperty(_T("highlight"), hlFile);
  spuNode->AddProperty(_T("select"), selFile);
  spuNode->AddProperty(_T("transparent"), SConv::ToString(m_transpColour, false));
  spuNode->AddProperty(_T("force"), _T("yes"));
  for (int i=0; i<(int)GetObjectsCount(); i++)
  {
    MenuObject* obj = GetObject(i);
	if (obj->IsButton())
	  spuNode->AddChild(obj->GetXML(SPUMUX_XML));
  }
  streamNode->AddChild(spuNode);
  root->AddChild(streamNode);
  xml.SetRoot(root);
  return xml.Save(fileName);
}

wxXmlNode* Menu::GetXML(DVDFileType type, wxXmlNode* node)
{
  if (node == NULL)
	node = new wxXmlNode(wxXML_ELEMENT_NODE, _T("menu"));
  
  if (type == DVDSTYLER_XML)
  {
	node->AddProperty(_T("videoFormat"), (m_videoFormat == vfPAL) ? _T("PAL") : _T("NTSC"));
	if (GetBackground().length())
	  node->AddProperty(_T("bgFile"), GetBackground());
	else
	  node->AddProperty(_T("bgColour"), SConv::ToString(GetBackgroundColour()));
    
    if (m_svg && m_svg->GetRoot())
    {
      wxSVGSVGElement* svgNode = (wxSVGSVGElement*) m_svg->GetRoot()->CloneNode();
      wxXmlElement* selRect = svgNode->GetElementById(wxT("selection"));
      if (selRect)
        selRect->GetParent()->RemoveChild(selRect);
      node->AppendChild(svgNode);
    }
  }
  
  for (int i=0; i<GetObjectsCount(); i++)
  {
	MenuObject* obj = GetObject(i);
	if (type == DVDSTYLER_XML || obj->IsButton())
	  node->AddChild(obj->GetXML(type));
  }
  
  return node;
}

bool Menu::PutXML(wxXmlNode* node)
{
  if (node->GetName() == _T("spumux"))
    node = node->GetChildren();
  if (node != NULL && node->GetName() == _T("menu"))
  {
    wxString val;
    m_videoFormat = vfPAL;
    if (node->GetPropVal(_T("videoFormat"), &val) && val == _T("NTSC"))
      m_videoFormat = vfNTSC;
      
    wxXmlNode* svgNode = XmlFindNode(node, wxT("svg"));
    if (svgNode)
    {
      wxXmlDocument xml;
      xml.SetRoot(svgNode->CloneNode());
      wxMemoryOutputStream output;
      xml.Save(output);
#if wxCHECK_VERSION(2,6,0)
      wxMemoryInputStream input(output);
      m_svg->Load(input);
#else
      char* data = new char[output.GetSize()];
      output.CopyTo(data, output.GetSize());
      wxMemoryInputStream input(data, output.GetSize());
      m_svg->Load(input);
      delete[] data;
#endif
    }
    else // deprecated
    {
      if (node->GetPropVal(_T("bgFile"), &val))
        SetBackground(val);
      else if (node->GetPropVal(_T("bgColour"), &val))
        SetBackgroundColour(SConv::ToColour(val));
    }
	
	for (wxXmlNode* child = node->GetChildren(); child != NULL; child = child->GetNext())
	  if (child->GetType() == wxXML_ELEMENT_NODE)
		AddObject(child);
  }
  return true;
}

wxString Menu::AddObject(wxXmlNode* node, bool fixPosition)
{
  if (node->GetName() != _T("button") && node->GetName() != _T("object"))
    return wxT("");
  
  MenuObject* obj = new MenuObject(this);
  if (!obj->PutXML(node))
  {
    delete obj;
	return wxT("");
  }
  if (fixPosition) // for copy & paste
	FixPosition(obj);
  m_objects.Add(obj);
  return obj->GetId();
}

void Menu::FixPosition(MenuObject* obj)
{
  if (obj->GetX() == -1)
    obj->SetX(GetResolution().x/4);
  if (obj->GetY() == -1)
    obj->SetY(GetResolution().y/4);
  while (obj->GetX()<GetResolution().x-48 && obj->GetY()<GetResolution().y-48)
  {
	bool found = false;
	for (int i=0; i<(int)GetObjectsCount(); i++)
	{
	  MenuObject* obj1 = GetObject(i);
	  if (obj1->GetX() == obj->GetX() &&
		  obj1->GetY() == obj->GetY())
	  {
		found = true;
		break;
	  }
	}
	if (!found)
	  break;
	obj->SetX(obj->GetX()+16);
    obj->SetY(obj->GetY()+16);
  }
}
